@@ -4,10 +4,10 @@ use strict;
 use warnings;
 
 use HTTP::Tiny;
-use JSON;
+use JSON::PP;
 use Data::CosineSimilarity;
 
-our $VERSION = '0.1_1';
+our $VERSION = '0.1_2';
 $VERSION = eval $VERSION;
 
 my $http = HTTP::Tiny->new;
@@ -17,13 +17,15 @@ sub new {
     my $class = shift;
     my %attr  = @_;
     
-    $attr{'error'}  = '';
+    $attr{'error'}      = '';
     
-    $attr{'api'}    = 'OpenAI' unless $attr{'api'};
-    $attr{'error'}  = 'Invalid API' unless $attr{'api'} eq 'OpenAI';
-    $attr{'error'}  = 'API Key missing' unless $attr{'key'};
+    $attr{'api'}        = 'OpenAI' unless $attr{'api'};
+    $attr{'error'}      = 'Invalid API' unless $attr{'api'} eq 'OpenAI';
+    $attr{'error'}      = 'API Key missing' unless $attr{'key'};
     
-    $attr{'model'}  = 'text-embedding-ada-002' unless $attr{'model'};
+    $attr{'model'}      = 'text-embedding-ada-002' unless $attr{'model'};
+    
+    $attr{'comparator'} = $class->_make_vector($attr{'comparator'}) if exists $attr{'comparator'};
     
     return bless \%attr, $class;
 }
@@ -35,7 +37,7 @@ my %url    = (
 
 # Define HTTP Headers for APIs
 my %header = (
-    'OpenAI' => \&_get_header_openai,
+    'OpenAI' => &_get_header_openai,
 );
 
 # Returns true if last operation was success
@@ -54,10 +56,8 @@ sub error {
 sub _get_header_openai {
     my $self = shift;
     return {
-        headers => {
-             'Authorization' => $self->key,
-             'Content-type'  => 'application/json'
-         }
+         'Authorization' => 'Bearer ' . $self->{'key'},
+         'Content-type'  => 'application/json'
      };
  }
  
@@ -66,7 +66,10 @@ sub _get_header_openai {
      my ($self, $text) = @_;
      
      return $http->post($url{$self->{'api'}}, {
-         $header{$self->{'api'}},
+         'headers' => {
+             'Authorization' => 'Bearer ' . $self->{'key'},
+             'Content-type'  => 'application/json'
+         },
          content => encode_json {
              input  => $text,
              model  => $self->{'model'},
@@ -74,6 +77,17 @@ sub _get_header_openai {
      });
  }
  
+ # TODO:
+ # Make 'headers' use $header{$self->{'api'}}
+ # Currently hard coded to OpenAI
+ 
+ # Added purely for testing - IGNORE!
+ sub test {
+     my $self = shift;
+#    return $self->{'api'};
+     return $header{$self->{'api'}};
+ }
+ 
  # Return Embedding as a CSV string
  sub embedding {
      my ($self, $text) = @_;
@@ -100,6 +114,23 @@ sub _get_header_openai {
      return $response;
  }
  
+ # Return Test Embedding
+ sub test_embedding {
+     my ($self, $text, $dimension) = @_;
+     
+     $dimension = 1536 unless defined $dimension;
+     
+     if ($text) {
+         srand scalar split /\s+/, $text;
+     }
+     
+     my @vector;
+     for (1...$dimension) {
+         push @vector, rand(2) - 1;
+     }
+     return join ',', @vector;
+ }
+ 
  # Convert a CSV Embedding into a hashref
  sub _make_vector {
      my ($self, $embed_string) = @_;
@@ -160,6 +191,8 @@ AI::Embedding - Perl module for working with text embeddings using various APIs
 
 Version 0.01
 
+This is currently a development release and may contain bugs!
+
 =head1 SYNOPSIS
 
     use AI::Embedding;
@@ -169,8 +202,9 @@ Version 0.01
         key => 'your-api-key'
     );
 
-    my $csv_embedding = $embedding->embedding('Some text');
-    my @raw_embedding = $embedding->raw_embedding('Some text');
+    my $csv_embedding  = $embedding->embedding('Some sample text');
+    my $test_embedding = $embedding->test_embedding('Some sample text');
+    my @raw_embedding  = $embedding->raw_embedding('Some sample text');
     $embedding->comparator($csv_embedding2);
 
     my $similarity = $embedding->compare($csv_embedding1);
@@ -180,11 +214,15 @@ Version 0.01
 
 The L<AI::Embedding> module provides an interface for working with text embeddings using various APIs. It currently supports the L<OpenAI|https://www.openai.com> L<Embeddings API|https://platform.openai.com/docs/guides/embeddings/what-are-embeddings>. This module allows you to generate embeddings for text, compare embeddings, and calculate cosine similarity between embeddings.
 
+Embeddings allow the meaning of passages of text to be compared for similarity.  This is more natural and useful to humans than using traditional keyword based comparisons.
+
 An Embedding is a multi-dimensional vector representing the meaning of a piece of text.  The Embedding vector is created by an AI Model.  The default model (OpenAI's C<text-embedding-ada-002>) produces a 1536 dimensional vector.  The resulting vector can be obtained as a Perl array or a Comma Separated String. As the Embedding will typically be used homogeneously, having it as a CSV String is usually more convenient.  This is suitable for storing in a C<TEXT> field of a database.
 
 =head2 Comparator
 
-Embeddings are used to compare similarity of meaning between two passages of text.  A typical work case is to store a number of pieces of text (e.g. articles or blogs) in a database and compare each one to some user supplied search text.  L<AI::Embedding> provides a C<compare> method to either compare two Embeddings or one Embedding to a previously supplied C<compatator>.  The C<comparator> can either be set when the object is constructed or by using the B<comparator> method.  When comparing multiple Embeddings to the same Embedding (such as search text) it is faster to use a C<comparator>.
+Embeddings are used to compare similarity of meaning between two passages of text.  A typical work case is to store a number of pieces of text (e.g. articles or blogs) in a database and compare each one to some user supplied search text.  L<AI::Embedding> provides a C<compare> method to either compare two Embeddings or one Embedding to a previously supplied C<compatator>.  The C<comparator> can either be set when the object is constructed or by using the B<comparator> method.  However, it is not usually available at the time tyhe object is created so will generally be supplied with the B<comparator> method.
+
+When comparing multiple Embeddings to the same Embedding (such as search text) it is faster to use a C<comparator>.
 
 =head1 CONSTRUCTOR
 
@@ -251,6 +289,12 @@ It is not normally necessary to use this method as the Embedding will almost alw
 
 If the method call fails it sets the L</"error"> message and returns the complete L<HTTP::Tiny> response object.
 
+=head2 raw_embedding
+
+    my @raw_embedding = $embedding->raw_embedding('Some text passage', $dimensions);
+    
+Used for testing code without making a chargable call to the API.  Returns a random embedding.  Both parameters are optional.  If a text string is provided, the returned embedding will always be the same random embedding otherwise it will random and different every time.  C<dimension> controls the number of elements of the returned CSV string.  If omitted, the string will have the C<text-embedding-ada-002> default of 1536 elements.
+
 =head2 comparator
 
     $embedding->comparator($csv_embedding2);
